@@ -1,4 +1,4 @@
-const Range = require('./util/Range');
+const { stringHints, numberHints } = require('./util/hints');
 
 /** @typedef {import("json-schema").JSONSchema6} JSONSchema6 */
 /** @typedef {import("json-schema").JSONSchema7} JSONSchema7 */
@@ -185,6 +185,42 @@ function indent(str, prefix) {
   return str.replace(/\n(?!$)/g, `\n${prefix}`);
 }
 
+/**
+ * @param {Schema} schema
+ * @returns {schema is (Schema & {not: Schema})}
+ */
+function hasNotInSchema(schema) {
+  return !!schema.not;
+}
+
+/**
+ * @param {Schema} schema
+ * @return {Schema}
+ */
+function findFirstTypedSchema(schema) {
+  if (hasNotInSchema(schema)) {
+    return findFirstTypedSchema(schema.not);
+  }
+
+  return schema;
+}
+
+/**
+ * @param {Schema} schema
+ * @return {boolean}
+ */
+function canApplyNot(schema) {
+  const typedSchema = findFirstTypedSchema(schema);
+
+  return (
+    likeNumber(typedSchema) ||
+    likeInteger(typedSchema) ||
+    likeString(typedSchema) ||
+    likeNull(typedSchema) ||
+    likeBoolean(typedSchema)
+  );
+}
+
 /**
  * @param {any} maybeObj
  * @returns {boolean}
@@ -224,7 +260,7 @@ function likeInteger(schema) {
 }
 
 /**
- * @param {Schema & {formatMinimum?: string; formatMaximum?: string;}} schema
+ * @param {Schema} schema
  * @returns {boolean}
  */
 function likeString(schema) {
@@ -332,47 +368,6 @@ function getSchemaNonTypes(schema) {
   return '';
 }
 
-/**
- * @param {Schema=} schema
- * @returns {Array<string>}
- */
-function numberHints(schema) {
-  if (!schema) {
-    return [];
-  }
-
-  const hints = [];
-  const range = new Range();
-
-  if (typeof schema.minimum === 'number') {
-    range.left(schema.minimum);
-  }
-
-  if (typeof schema.exclusiveMinimum === 'number') {
-    range.left(schema.exclusiveMinimum, true);
-  }
-
-  if (typeof schema.maximum === 'number') {
-    range.right(schema.maximum);
-  }
-
-  if (typeof schema.exclusiveMaximum === 'number') {
-    range.right(schema.exclusiveMaximum, true);
-  }
-
-  const rangeFormat = range.format();
-
-  if (rangeFormat) {
-    hints.push(rangeFormat);
-  }
-
-  if (typeof schema.multipleOf === 'number') {
-    hints.push(`should be multiple of ${schema.multipleOf}`);
-  }
-
-  return hints;
-}
-
 /**
  * @param {Array<string>} hints
  * @returns {string}
@@ -383,14 +378,17 @@ function formatHints(hints) {
 
 /**
  * @param {Schema} schema
- * @returns {string}
+ * @param {boolean} logic
+ * @returns {string[]}
  */
-function getHints(schema) {
+function getHints(schema, logic) {
   if (likeNumber(schema) || likeInteger(schema)) {
-    return formatHints(numberHints(schema));
+    return numberHints(schema, logic);
+  } else if (likeString(schema)) {
+    return stringHints(schema, logic);
   }
 
-  return '';
+  return [];
 }
 
 class ValidationError extends Error {
@@ -471,10 +469,12 @@ class ValidationError extends Error {
 
   /**
    * @param {Schema} schema
+   * @param {boolean} logic
    * @param {Array<Object>} prevSchemas
    * @returns {string}
    */
-  formatSchema(schema, prevSchemas = []) {
+  formatSchema(schema, logic = true, prevSchemas = []) {
+    let newLogic = logic;
     const formatInnerSchema =
       /**
        *
@@ -484,22 +484,39 @@ class ValidationError extends Error {
        */
       (innerSchema, addSelf) => {
         if (!addSelf) {
-          return this.formatSchema(innerSchema, prevSchemas);
+          return this.formatSchema(innerSchema, newLogic, prevSchemas);
         }
 
         if (prevSchemas.includes(innerSchema)) {
           return '(recursive)';
         }
 
-        return this.formatSchema(innerSchema, prevSchemas.concat(schema));
+        return this.formatSchema(
+          innerSchema,
+          newLogic,
+          prevSchemas.concat(schema)
+        );
       };
 
-    if (schema.not && !likeObject(schema)) {
-      return `non ${formatInnerSchema(schema.not)}`;
+    if (hasNotInSchema(schema) && !likeObject(schema)) {
+      if (canApplyNot(schema.not)) {
+        newLogic = !logic;
+
+        return formatInnerSchema(schema.not);
+      }
+
+      const needApplyLogicHere = !schema.not.not;
+      const prefix = logic ? '' : 'non ';
+      newLogic = !logic;
+
+      return needApplyLogicHere
+        ? prefix + formatInnerSchema(schema.not)
+        : formatInnerSchema(schema.not);
     }
 
     if (
-      /** @type {Schema & {instanceof: string | Array<string>}} */ (schema).instanceof
+      /** @type {Schema & {instanceof: string | Array<string>}} */ (schema)
+        .instanceof
     ) {
       const {
         instanceof: value,
@@ -563,85 +580,34 @@ class ValidationError extends Error {
     }
 
     if (likeNumber(schema) || likeInteger(schema)) {
-      const type = schema.type === 'integer' ? 'integer' : 'number';
-      const hints = getHints(schema);
+      const [type, ...hints] = getHints(schema, logic);
+      const str = `${type}${hints.length > 0 ? ` ${formatHints(hints)}` : ''}`;
 
-      return `${type}${hints.length > 0 ? ` ${hints}` : ''}`;
+      return logic
+        ? str
+        : hints.length > 0
+        ? `non-${type} | ${str}`
+        : `non-${type}`;
     }
 
     if (likeString(schema)) {
-      let type = 'string';
-      const hints = [];
+      const [type, ...hints] = getHints(schema, logic);
+      const str = `${type}${hints.length > 0 ? ` ${formatHints(hints)}` : ''}`;
 
-      if (typeof schema.minLength === 'number') {
-        if (schema.minLength === 1) {
-          type = 'non-empty string';
-        } else if (schema.minLength !== 0) {
-          /* if min length === 0 it does not make hint for user */
-          const length = schema.minLength - 1;
-
-          hints.push(
-            `should be longer than ${length} character${length > 1 ? 's' : ''}`
-          );
-        }
-      }
-
-      if (typeof schema.maxLength === 'number') {
-        if (schema.maxLength === 0) {
-          type = 'empty string';
-        } else {
-          hints.push(
-            `should be shorter than ${schema.maxLength + 1} characters`
-          );
-        }
-      }
-
-      if (schema.pattern) {
-        hints.push(`should match pattern ${JSON.stringify(schema.pattern)}`);
-      }
-
-      if (schema.format) {
-        hints.push(`should match format ${JSON.stringify(schema.format)}`);
-      }
-
-      if (
-        /** @type {Schema & {formatMinimum?: string; formatExclusiveMinimum?: boolean;}} */ (schema).formatMinimum
-      ) {
-        const {
-          formatExclusiveMinimum,
-          formatMinimum,
-        } = /** @type {Schema & {formatMinimum?: string; formatExclusiveMinimum?: boolean;}} */ (schema);
-
-        hints.push(
-          `should be ${formatExclusiveMinimum ? '>' : '>='} ${JSON.stringify(
-            formatMinimum
-          )}`
-        );
-      }
-
-      if (
-        /** @type {Schema & {formatMaximum?: string; formatExclusiveMaximum?: boolean;}} */ (schema).formatMaximum
-      ) {
-        const {
-          formatExclusiveMaximum,
-          formatMaximum,
-        } = /** @type {Schema & {formatMaximum?: string; formatExclusiveMaximum?: boolean;}} */ (schema);
-
-        hints.push(
-          `should be ${formatExclusiveMaximum ? '<' : '<='} ${JSON.stringify(
-            formatMaximum
-          )}`
-        );
-      }
-
-      return `${type}${hints.length > 0 ? ` (${hints.join(', ')})` : ''}`;
+      return logic
+        ? str
+        : str === 'string'
+        ? 'non-string'
+        : `non-string | ${str}`;
     }
 
     if (likeBoolean(schema)) {
-      return 'boolean';
+      return `${logic ? '' : 'non-'}boolean`;
     }
 
     if (likeArray(schema)) {
+      // not logic already applied in formatValidationError
+      newLogic = true;
       const hints = [];
 
       if (typeof schema.minItems === 'number') {
@@ -716,6 +682,8 @@ class ValidationError extends Error {
     }
 
     if (likeObject(schema)) {
+      // not logic already applied in formatValidationError
+      newLogic = true;
       const hints = [];
 
       if (typeof schema.minProperties === 'number') {
@@ -834,14 +802,16 @@ class ValidationError extends Error {
     }
 
     if (likeNull(schema)) {
-      return 'null';
+      return `${logic ? '' : 'non-'}null`;
     }
 
     if (Array.isArray(schema.type)) {
+      // not logic already applied in formatValidationError
       return `${schema.type.join(' | ')}`;
     }
 
     // Fallback for unknown keywords
+    // not logic already applied in formatValidationError
     /* istanbul ignore next */
     return JSON.stringify(schema, null, 2);
   }
@@ -850,9 +820,10 @@ class ValidationError extends Error {
    * @param {Schema=} schemaPart
    * @param {(boolean | Array<string>)=} additionalPath
    * @param {boolean=} needDot
+   * @param {boolean=} logic
    * @returns {string}
    */
-  getSchemaPartText(schemaPart, additionalPath, needDot = false) {
+  getSchemaPartText(schemaPart, additionalPath, needDot = false, logic = true) {
     if (!schemaPart) {
       return '';
     }
@@ -877,7 +848,9 @@ class ValidationError extends Error {
       schemaPart = this.getSchemaPart(schemaPart.$ref);
     }
 
-    let schemaText = `${this.formatSchema(schemaPart)}${needDot ? '.' : ''}`;
+    let schemaText = `${this.formatSchema(schemaPart, logic)}${
+      needDot ? '.' : ''
+    }`;
 
     if (schemaPart.description) {
       schemaText += `\n-> ${schemaPart.description}`;
@@ -1017,7 +990,10 @@ class ValidationError extends Error {
           comparison,
           limit,
         } = /** @type {import("ajv").ComparisonParams} */ (params);
-        const hints = numberHints(parentSchema);
+        const [, ...hints] = getHints(
+          /** @type {Schema} */ (parentSchema),
+          true
+        );
 
         if (hints.length === 0) {
           hints.push(`should be ${comparison} ${limit}`);
@@ -1099,11 +1075,13 @@ class ValidationError extends Error {
       case 'maxLength': {
         const { params, parentSchema } = error;
         const { limit } = /** @type {import("ajv").LimitParams} */ (params);
+        const max = limit + 1;
 
-        return `${dataPath} should be shorter than ${limit +
-          1} characters${getSchemaNonTypes(
+        return `${dataPath} should be shorter than ${max} character${
+          max > 1 ? 's' : ''
+        }${getSchemaNonTypes(parentSchema)}.${this.getSchemaPartDescription(
           parentSchema
-        )}.${this.getSchemaPartDescription(parentSchema)}`;
+        )}`;
       }
       case 'maxItems': {
         const { params, parentSchema } = error;
@@ -1249,6 +1227,20 @@ class ValidationError extends Error {
         )}`;
       }
       case 'not': {
+        const postfix = likeObject(/** @type {Schema} */ (error.parentSchema))
+          ? `\n${this.getSchemaPartText(error.parentSchema)}`
+          : '';
+        const schemaOutput = this.getSchemaPartText(
+          error.schema,
+          false,
+          false,
+          false
+        );
+
+        if (canApplyNot(error.schema)) {
+          return `${dataPath} should be any ${schemaOutput}${postfix}.`;
+        }
+
         const { schema, parentSchema } = error;
 
         return `${dataPath} should not be ${this.getSchemaPartText(
